/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.programmation-facile.com/
  Portions created by the Initial Developer are Copyright (C) 2013
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

 */



/**
 * Le Framework de jeux JavaScript Phaser
 *
Phaser v2.0.5 - http://phaser.io - @photonstorm - (c) 2014 Photon Storm Ltd. 
https://github.com/photonstorm/phaser
http://examples.phaser.io/
 */



/**
 * Les variables du jeu
 * 
 */

var game;

var tileSize = 100;// taille d'une case

// talbeau du jeu
var fieldArray = new Array(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);

var tileSpritesGroup;// groupe contenant tous les sprites

// les couleurs des cases en fonction de leurs valeurs
var aColorsList = {
    2:0xFFFFFF,
    4:0xFFEEEE,
    8:0xFFDDDD,
    16:0xFFCCCC,
    32:0xFFBBBB,
    64:0xFFAAAA,
    128:0xFF9999,
    256:0xFF8888,
    512:0xFF7777,
    1024:0xFF6666,
    2048:0xFF5555
}

var bPlayerCanMove=false;// au début du jeu, le joueur ne peut rien déplacer



document.addEventListener("DOMContentLoaded", Main, false); // appel au chargement de la page


/**
 * Fonction principale
 * Appelée au chargement de la page
 * 
 */
function Main()
{
    console.log("Main");

    // creation de la zone de jeu - API Canvas
    game = new Phaser.Game(tileSize*4, tileSize*4, Phaser.AUTO, "idGameDiv", {preload:onPreload, create:onCreate, update:onUpdate});
}


/**
 * Préchargement des éléments du jeu
 * 
 * @return {[type]} [description]
 */
function onPreload()
{ 
    game.stage.backgroundColor = '#cccccc';// la couleur de fond du jeu
    game.load.image("tile", "assets/tile.png");// les éléments graphiques avec leurs identifiants
}



/**
 * Création de la scène du jeu,
 * mise en place des graphismes...
 * 
 * @return {[type]} [description]
 */
function onCreate() 
{
    // écoute les touches directionnelles pressées par le joueur
    var cursors = game.input.keyboard.createCursorKeys();// controle du héros avec les touches directionnelles

    cursors.left.onDown.add(moveLeft, this);
    cursors.right.onDown.add(moveRight, this);
    cursors.up.onDown.add(moveUp, this);
    cursors.down.onDown.add(moveDown, this);

    // groupe contenant toutes les cases
    tileSpritesGroup = game.add.group();
    
    // au début du jeu, ajoute 2 cases
    addCaseValTwo();
    addCaseValTwo();
}


/**
 * Ajoute une case de valeur 2
 * 
 */
function addCaseValTwo()
{
    // choisis une case vide sur la zone de jeu
    do
    {
        var randomValue = Math.floor(Math.random()*16);
    }
    while (fieldArray[randomValue] != 0)// tant que la case vide n'est pas trouvée

    fieldArray[randomValue]=2;// valeur de la case = 2

    // creation graphique de la case
    var tile = game.add.sprite( getValueColonne(randomValue) * tileSize, getValueRow(randomValue)  *tileSize, "tile");

    tile.pos = randomValue;// sauvegarde la valeur de la case
    tile.alpha = 0;// au départ la case est transparente (invisible)

    // ajoute le texte avec la valeur de la case
    var text = game.add.text(tileSize/2, tileSize/2, "2", {font:"bold 33px Arial", align:"center"});
    text.anchor.set(0.5);// mise en forme du texte

    tile.addChild(text);// ajoute le texte par dessus la case (enfant)
    tileSpritesGroup.add(tile);// ajotue la case dans le groupe
    
    // création d'un effet tween pour la case
    var fadeIn = game.add.tween(tile);    
    fadeIn.to({alpha:1},250);// la case apparaît au bout de 250 milliseconds
    
    // appelle de cette fonction à la fin de la tween
    fadeIn.onComplete.add(function()
    {
        updateCaseValueAndColor();// mets à jour la valeur de la case. Inutile la première fois, pour plus tard.
        bPlayerCanMove = true;// le joueur peut déplacer les cases
    })

    fadeIn.start();// lance la tween
}


/**
 * Fournit une valeur : la ligne du tableau
 * 
 * @param  {[type]} nValue [description]
 * @return {[type]}   [description]
 */
function getValueRow(nValue)
{
    return Math.floor(nValue/4);
}


/**
 * Fournit une valeur : la colonne du tableau
 * 
 * @param  {[type]} nValue [description]
 * @return {[type]}   [description]
 */
function getValueColonne(nValue)
{
    return nValue%4; 
}


/**
 * Mets à jour la valeur et la couleur de la case
 * @return {[type]} [description]
 */
function updateCaseValueAndColor()
{
    // parcours toutes les cases
    tileSpritesGroup.forEach(function(item)
    {
        var value = fieldArray[item.pos];// récupère la valeur de la case
        item.getChildAt(0).text = value;// mets à jour le texte de la case
        item.tint = aColorsList[value];// modifie la couleur de la case
    }); 
}


/**
 * Déplace les cases vers la gauche
 * 
 * @return {[type]} [description]
 */
function moveLeft()
{
    // si le joueur peut déplacer des cases
    if(bPlayerCanMove)
    {
        bPlayerCanMove = false;// un seul déplacement à la fois, empêche le joueur de se déplacer

        var moved = false;// conserve en mémoire si le joueur se déplace et si c'est possible

        tileSpritesGroup.sort("x", Phaser.Group.SORT_ASCENDING);// trie des cases suivant une propriété
        
        // parcours toutes les cases
        tileSpritesGroup.forEach(function(item)
        {
            // récupère la colonne et la ligne de la case
            var row = getValueRow(item.pos);
            var col = getValueColonne(item.pos);

            // si la case n'est pas déjà sur le bord de la colonne de gauche sinon impossible de la déplacer vers la gauche
            if(col>0)
            {
                var remove = false;// mets le drapeau "suppression" à faux de la case si elle fusionne avec une autre

                // boucle sur la position de la colonne la plus à gauche (pour la détecter)
                for(i=col-1;i>=0;i--)
                {
                    // si nous trouvons une case pleine alors nous avons terminé la recherche
                    if(fieldArray[row*4+i]!=0)
                    {

                        // vérification de la valeur de la case d'arrivée (si c'est la même valeur)
                        if(fieldArray[row*4+i]==fieldArray[row*4+col])
                        {
                            // oui, mise à true du drapeau suppression de la case
                            remove = true;
                            i--;                                             
                        }
                        break;
                    }
                }

                // si c'est impossible de se déplacer
                if(col!=i+1)
                {
                    moved=true;// déplacement possible
                    moveTile(item,row*4+col,row*4+i+1, remove);// déplace la case et suppression si drapeau à true
                }
            }
        });

        endMove(moved);// termine le déplacement
    }
}



/**
 * Termine le déplacement
 * et ajoute une case de valeur 2 si c'est possible
 * 
 * @param  {[type]} bAddCase [description]
 * @return {[type]}   [description]
 */
function endMove(bAddCase)
{
    // si la case a été déplacée
    if(bAddCase)    
        addCaseValTwo();
    else
        bPlayerCanMove=true;// le joueur peut se déplacer maintenant
}


/**
 * Déplacement d'une case
 * 
 * @param  {[type]} tile   [description]
 * @param  {[type]} from   [description]
 * @param  {[type]} to     [description]
 * @param  {[type]} remove [description]
 * @return {[type]}        [description]
 */
function moveTile(tile, from, to, remove)
{
    // mise à jour du tableau avec les nouvelles valeurs
    fieldArray[to]=fieldArray[from];
    fieldArray[from]=0;
    tile.pos=to;
    
    // création d'une tween
    var movement = game.add.tween(tile);
    movement.to({x:tileSize*(getValueColonne(to)),y:tileSize*(getValueRow(to))},150);
   
    // si suppression de la case
    if(remove)
    {
        fieldArray[to]*=2;// la valeur de la case de destination est multipliée par deux
        
        // à la fin de la tween, suppression de la case
        movement.onComplete.add(function()
        {
            tile.destroy();
        });
    }
    
    movement.start();// lance l'effet de tween
}



/**
 * Déplace les cases vers le haut
 * (même principe que précédemment)
 * 
 * @return {[type]} [description]
 */
function moveUp()
{
    if(bPlayerCanMove)
    {
        bPlayerCanMove=false;
        var moved=false;

        tileSpritesGroup.sort("y",Phaser.Group.SORT_ASCENDING);

        tileSpritesGroup.forEach(function(item)
        {
            var row = getValueRow(item.pos);
            var col = getValueColonne(item.pos);
            
            if(row>0)
            {  
                var remove=false;

                for(i=row-1;i>=0;i--)
                {
                    if(fieldArray[i*4+col]!=0)
                    {
                        if(fieldArray[i*4+col]==fieldArray[row*4+col])
                        {
                            remove = true;
                            i--;                                             
                        }
                        
                        break;// sortie de la boucle
                    }
                }

                if(row!=i+1)
                {
                     moved=true;
                     moveTile(item,row*4+col,(i+1)*4+col,remove);
                }
            }
        });

        endMove(moved);
    }
}


/**
 * Déplace les cases vers la droite
 * (même principe que précédemment)
 * 
 * @return {[type]} [description]
 */
function moveRight()
{
    if(bPlayerCanMove)
    {
        bPlayerCanMove=false;
        var moved=false;

        tileSpritesGroup.sort("x",Phaser.Group.SORT_DESCENDING);

        tileSpritesGroup.forEach(function(item)
        {
            var row = getValueRow(item.pos);
            var col = getValueColonne(item.pos);
            
            if(col<3)
            {
                var remove = false;

                for(i=col+1;i<=3;i++)
                {
                    if(fieldArray[row*4+i]!=0)
                    {
                        if(fieldArray[row*4+i]==fieldArray[row*4+col])
                        {
                            remove = true;
                            i++;                                             
                        }
                        break;
                    }
                }

                if(col!=i-1)
                {
                    moved=true;
                    moveTile(item,row*4+col,row*4+i-1,remove);
                }
            }
        });
        endMove(moved);
    }
}


/**
 * Déplace les cases vers le bas
 * (même principe que précédemment)
 * 
 * @return {[type]} [description]
 */
function moveDown()
{
    if(bPlayerCanMove)
    {
        bPlayerCanMove=false;
        var moved=false;
        
        tileSpritesGroup.sort("y",Phaser.Group.SORT_DESCENDING);

        tileSpritesGroup.forEach(function(item)
        {
            var row = getValueRow(item.pos);
            var col = getValueColonne(item.pos);

            if(row<3)
            {
                var remove = false;

                for(i=row+1;i<=3;i++)
                {
                    if(fieldArray[i*4+col]!=0)
                    {
                        if(fieldArray[i*4+col]==fieldArray[row*4+col])
                        {
                            remove = true;
                            i++;                                             
                        }
                        break;
                    }
                }

                if(row!=i-1)
                {
                    moved=true;
                    moveTile(item,row*4+col,(i-1)*4+col,remove);
                }
            }
        });

        endMove(moved);
    }
}



/**
 * Appelée en continu pendant le jeu
 * 
 * @return {[type]} [description]
 */
function onUpdate() 
{
    // mise à jour du score...
}

